@@ -1,7 +1,7 @@
 package App::lcpan;
 
-our $DATE = '2015-06-17'; # DATE
-our $VERSION = '0.41'; # VERSION
+our $DATE = '2015-06-28'; # DATE
+our $VERSION = '0.42'; # VERSION
 
 use 5.010001;
 use strict;
@@ -81,6 +81,24 @@ our %query_args = (
     },
 );
 
+our %query_multi_args = (
+    query => {
+        summary => 'Search query',
+        schema => ['array*', of=>'str*'],
+        cmdline_aliases => {q=>{}},
+        pos => 0,
+        greedy => 1,
+    },
+    detail => {
+        schema => 'bool',
+    },
+    exact_match => {
+        summary => 'Match query with exact module names',
+        schema => 'bool',
+        default => 0,
+    },
+);
+
 our %fauthor_args = (
     author => {
         summary => 'Filter by author',
@@ -1310,7 +1328,7 @@ $SPEC{authors} = {
     summary => 'List authors',
     args => {
         %common_args,
-        %query_args,
+        %query_multi_args,
     },
     result => {
         description => <<'_',
@@ -1341,16 +1359,28 @@ sub authors {
     my $cpan = $args{cpan};
     my $index_name = $args{index_name};
     my $detail = $args{detail};
-    my $q = $args{query} // ''; # sqlite is case-insensitive by default, yay
-    $q = '%'.$q.'%' unless $q =~ /%/;
 
     my $dbh = _connect_db('ro', $cpan, $index_name);
 
     my @bind;
     my @where;
-    if (length($q)) {
-        push @where, "(cpanid LIKE ? OR fullname LIKE ? OR email like ?)";
-        push @bind, $q, $q, $q;
+    {
+        my @q_where;
+        for my $q0 (@{ $args{query} // [] }) {
+            if ($args{exact_match}) {
+                push @q_where, "(cpanid=?)";
+                push @bind, uc($q0);
+            } else {
+                my $q = uc($q0 =~ /%/ ? $q0 : '%'.$q0.'%');
+                push @q_where, "(cpanid LIKE ? OR fullname LIKE ? OR email like ?)";
+                push @bind, $q, $q, $q;
+            }
+        }
+        if (@q_where > 1) {
+            push @where, "(".join(" OR ", @q_where).")";
+        } elsif (@q_where == 1) {
+            push @where, @q_where;
+        }
     }
     my $sql = "SELECT
   cpanid id,
@@ -1377,7 +1407,7 @@ $SPEC{modules} = {
     summary => 'List modules/packages',
     args => {
         %common_args,
-        %query_args,
+        %query_multi_args,
         %fauthor_args,
         %fdist_args,
         %flatest_args,
@@ -1409,8 +1439,6 @@ sub modules {
     my $cpan = $args{cpan};
     my $index_name = $args{index_name};
     my $detail = $args{detail};
-    my $q = $args{query} // ''; # sqlite is case-insensitive by default, yay
-    $q = '%'.$q.'%' unless $q =~ /%/;
     my $author = uc($args{author} // '');
 
     my $dbh = _connect_db('ro', $cpan, $index_name);
@@ -1442,10 +1470,24 @@ sub modules {
 
     my @bind;
     my @where;
-    if (length($q)) {
-        #push @where, "(name LIKE ? OR dist LIKE ?)"; # rather slow
-        push @where, "(name LIKE ? OR abstract LIKE ?)";
-        push @bind, $q, $q;
+    {
+        my @q_where;
+        for my $q0 (@{ $args{query} // [] }) {
+            #push @q_where, "(name LIKE ? OR dist LIKE ?)"; # rather slow
+            if ($args{exact_match}) {
+                push @q_where, "(name=?)";
+                push @bind, $q0;
+            } else {
+                my $q = $q0 =~ /%/ ? $q0 : '%'.$q0.'%';
+                push @q_where, "(name LIKE ? OR abstract LIKE ?)";
+                push @bind, $q, $q;
+            }
+        }
+        if (@q_where > 1) {
+            push @where, "(".join(" OR ", @q_where).")";
+        } elsif (@q_where == 1) {
+            push @where, @q_where;
+        }
     }
     if ($author) {
         push @where, "(author=?)";
@@ -1459,7 +1501,7 @@ sub modules {
     if ($args{namespace}) {
         return [400, "Invalid namespace, please use Word or Word(::Sub)+"]
             unless $args{namespace} =~ /\A\w+(::\w+)*\z/;
-        push @where, "(name='$args{namespace}' OR name LIKE '$args{namespace}%')";
+        push @where, "(name='$args{namespace}' OR name LIKE '$args{namespace}::%')";
     }
     if ($args{latest}) {
         push @where, "(SELECT is_latest FROM dist d WHERE d.file_id=module.file_id)";
@@ -1492,7 +1534,7 @@ $SPEC{dists} = {
     summary => 'List distributions',
     args => {
         %common_args,
-        %query_args,
+        %query_multi_args,
         %fauthor_args,
         %flatest_args,
         has_makefilepl => {
@@ -1551,17 +1593,29 @@ sub dists {
     my $cpan = $args{cpan};
     my $index_name = $args{index_name};
     my $detail = $args{detail};
-    my $q = $args{query} // '';
-    $q = '%'.$q.'%' unless $q =~ /%/;
     my $author = uc($args{author} // '');
 
     my $dbh = _connect_db('ro', $cpan, $index_name);
 
     my @bind;
     my @where;
-    if (length($q)) {
-        push @where, "(d.name LIKE ? OR abstract LIKE ?)";
-        push @bind, $q, $q;
+    {
+        my @q_where;
+        for my $q0 (@{ $args{query} // [] }) {
+            if ($args{exact_match}) {
+                push @q_where, "(d.name=?)";
+                push @bind, $q0;
+            } else {
+                my $q = $q0 =~ /%/ ? $q0 : '%'.$q0.'%';
+                push @q_where, "(d.name LIKE ? OR abstract LIKE ?)";
+                push @bind, $q, $q;
+            }
+        }
+        if (@q_where > 1) {
+            push @where, "(".join(" OR ", @q_where).")";
+        } elsif (@q_where == 1) {
+            push @where, @q_where;
+        }
     }
     if ($author) {
         push @where, "(author=?)";
@@ -1630,7 +1684,7 @@ $SPEC{'releases'} = {
     args => {
         %common_args,
         %fauthor_args,
-        %query_args,
+        %query_multi_args,
         has_metajson   => {schema=>'bool'},
         has_metayml    => {schema=>'bool'},
         has_makefilepl => {schema=>'bool'},
@@ -1656,17 +1710,29 @@ sub releases {
     my $cpan = $args{cpan};
     my $index_name = $args{index_name};
     my $detail = $args{detail};
-    my $q = $args{query} // ''; # sqlite is case-insensitive by default, yay
-    $q = '%'.$q.'%' unless $q =~ /%/;
     my $author = uc($args{author} // '');
 
     my $dbh = _connect_db('ro', $cpan, $index_name);
 
     my @bind;
     my @where;
-    if (length($q)) {
-        push @where, "(f1.name LIKE ?)";
-        push @bind, $q;
+    {
+        my @q_where;
+        for my $q0 (@{ $args{query} // [] }) {
+            if ($args{exact_match}) {
+                push @q_where, "(f1.name=?)";
+                push @bind, $q0;
+            } else {
+                my $q = $q0 =~ /%/ ? $q0 : '%'.$q0.'%';
+                push @q_where, "(f1.name LIKE ?)";
+                push @bind, $q;
+            }
+        }
+        if (@q_where > 1) {
+            push @where, "(".join(" OR ", @q_where).")";
+        } elsif (@q_where == 1) {
+            push @where, @q_where;
+        }
     }
     if ($author) {
         push @where, "(f1.cpanid=?)";
@@ -2122,7 +2188,7 @@ $SPEC{namespaces} = {
     summary => 'List namespaces',
     args => {
         %common_args,
-        %query_args,
+        %query_multi_args,
         from_level => {
             schema => ['int*', min=>0],
             tags => ['category:filtering'],
@@ -2138,6 +2204,7 @@ $SPEC{namespaces} = {
         },
         sort => {
             schema => ['str*', in=>[qw/name -name num_modules -num_modules/]],
+            default => 'name',
             tags => ['category:sorting'],
         },
     },
@@ -2149,16 +2216,28 @@ sub namespaces {
     my $cpan = $args{cpan};
     my $index_name = $args{index_name};
     my $detail = $args{detail};
-    my $q = $args{query} // ''; # sqlite is case-insensitive by default, yay
-    $q = '%'.$q.'%' unless $q =~ /%/;
 
     my $dbh = _connect_db('ro', $cpan, $index_name);
 
     my @bind;
     my @where;
-    if (length($q)) {
-        push @where, "(name LIKE ?)";
-        push @bind, $q;
+    {
+        my @q_where;
+        for my $q0 (@{ $args{query} // [] }) {
+            if ($args{exact_match}) {
+                push @q_where, "(name=?)";
+                push @bind, $q0;
+            } else {
+                my $q = $q0 =~ /%/ ? $q0 : '%'.$q0.'%';
+                push @q_where, "(name LIKE ?)";
+                push @bind, $q;
+            }
+        }
+        if (@q_where > 1) {
+            push @where, "(".join(" OR ", @q_where).")";
+        } elsif (@q_where == 1) {
+            push @where, @q_where;
+        }
     }
     if (defined $args{from_level}) {
         push @where, "(num_sep >= ?)";
@@ -2212,7 +2291,7 @@ App::lcpan - Manage your local CPAN mirror
 
 =head1 VERSION
 
-This document describes version 0.41 of App::lcpan (from Perl distribution App-lcpan), released on 2015-06-17.
+This document describes version 0.42 of App::lcpan (from Perl distribution App-lcpan), released on 2015-06-28.
 
 =head1 SYNOPSIS
 
@@ -2248,7 +2327,7 @@ Examples:
 List all authors.
 
 
- authors( query => "MICHAEL%"); # -> ["MICHAEL", "MICHAELW"]
+ authors( query => ["MICHAEL%"]); # -> ["MICHAEL", "MICHAELW"]
 
 
 Find CPAN IDs which start with something.
@@ -2266,11 +2345,15 @@ Defaults to C<~/cpan>.
 
 =item * B<detail> => I<bool>
 
+=item * B<exact_match> => I<bool> (default: 0)
+
+Match query with exact module names.
+
 =item * B<index_name> => I<str> (default: "index.db")
 
 Filename of index.
 
-=item * B<query> => I<str>
+=item * B<query> => I<array[str]>
 
 Search query.
 
@@ -2372,7 +2455,7 @@ List all distributions.
 List all distributions (latest version only).
 
 
- dists( cpan => "/cpan", query => "data-table");
+ dists( cpan => "/cpan", query => ["data-table"]);
 
 
 Grep by distribution name, return detailed record.
@@ -2400,6 +2483,10 @@ Defaults to C<~/cpan>.
 
 =item * B<detail> => I<bool>
 
+=item * B<exact_match> => I<bool> (default: 0)
+
+Match query with exact module names.
+
 =item * B<has_buildpl> => I<bool>
 
 =item * B<has_makefilepl> => I<bool>
@@ -2414,7 +2501,7 @@ Filename of index.
 
 =item * B<latest> => I<bool>
 
-=item * B<query> => I<str>
+=item * B<query> => I<array[str]>
 
 Search query.
 
@@ -2460,6 +2547,10 @@ Defaults to C<~/cpan>.
 
 Filter by distribution.
 
+=item * B<exact_match> => I<bool> (default: 0)
+
+Match query with exact module names.
+
 =item * B<index_name> => I<str> (default: "index.db")
 
 Filename of index.
@@ -2470,7 +2561,7 @@ Filename of index.
 
 Select modules belonging to certain namespace.
 
-=item * B<query> => I<str>
+=item * B<query> => I<array[str]>
 
 Search query.
 
@@ -2512,6 +2603,10 @@ Defaults to C<~/cpan>.
 
 =item * B<detail> => I<bool>
 
+=item * B<exact_match> => I<bool> (default: 0)
+
+Match query with exact module names.
+
 =item * B<from_level> => I<int>
 
 =item * B<index_name> => I<str> (default: "index.db")
@@ -2520,11 +2615,11 @@ Filename of index.
 
 =item * B<level> => I<int>
 
-=item * B<query> => I<str>
+=item * B<query> => I<array[str]>
 
 Search query.
 
-=item * B<sort> => I<str>
+=item * B<sort> => I<str> (default: "name")
 
 =item * B<to_level> => I<int>
 
@@ -2566,6 +2661,10 @@ Defaults to C<~/cpan>.
 
 Filter by distribution.
 
+=item * B<exact_match> => I<bool> (default: 0)
+
+Match query with exact module names.
+
 =item * B<index_name> => I<str> (default: "index.db")
 
 Filename of index.
@@ -2576,7 +2675,7 @@ Filename of index.
 
 Select modules belonging to certain namespace.
 
-=item * B<query> => I<str>
+=item * B<query> => I<array[str]>
 
 Search query.
 
@@ -2685,6 +2784,10 @@ Defaults to C<~/cpan>.
 
 =item * B<detail> => I<bool>
 
+=item * B<exact_match> => I<bool> (default: 0)
+
+Match query with exact module names.
+
 =item * B<full_path> => I<bool>
 
 =item * B<has_buildpl> => I<bool>
@@ -2701,7 +2804,7 @@ Filename of index.
 
 =item * B<latest> => I<bool>
 
-=item * B<query> => I<str>
+=item * B<query> => I<array[str]>
 
 Search query.
 
